local Graph=AceLibrary("Graph-1.0")
local me={}
local FreeWindows={}
local WindowNum=1

function me:ResizeRealtimeWindow()
	self.Graph:SetWidth(self:GetWidth()-3)
	self.Graph:SetHeight(self:GetHeight()-33)
	self:UpdateTitle()
end

local Log2=math.log(2)


function me:DetermineGridSpacing()
	local MaxValue=self.Graph:GetMaxValue()
	local Spacing,Inbetween

	if MaxValue<50 then
		Spacing=-1
	else
		Spacing=math.log(MaxValue/100)/Log2
	end

	Inbetween=math.ceil(Spacing)-Spacing
	
	if Inbetween==0 then
		Inbetween=1
	end

	Spacing=25*math.pow(2,math.floor(Spacing))
	
	self.Graph:SetGridSpacing(1.0,Spacing)
	self.Graph:SetGridColorSecondary({0.5,0.5,0.5,0.5*Inbetween})
end

function me:UpdateTitle()
	self:DetermineGridSpacing()
	

	local Width,StartText, EndText
	Width=self:GetWidth()-32
	StartText=self.TitleText
	EndText=" - "..string.format("%.1f",self.Graph:GetValue(-0.05))

	self.Title:SetText(StartText..EndText)

	while self.Title:GetStringWidth()>Width do
		StartText=strsub(StartText,1,#StartText-1)
		self.Title:SetText(StartText.."..."..EndText)
	end
end

function me:SavePosition()
	local point,relativeTo,relativePoint,xOfs,yOfs = self:GetPoint(1)

	Recount.db.char.RealtimeWindows[self.id][4]=xOfs
	Recount.db.char.RealtimeWindows[self.id][5]=yOfs
	Recount.db.char.RealtimeWindows[self.id][6]=self:GetWidth()
	Recount.db.char.RealtimeWindows[self.id][7]=self:GetHeight()
end

function me:FreeWindow()
	Recount:UnregisterTracking(this.id,this.who,this.tracking)
	table.insert(FreeWindows,this)
	Recount:CancelScheduledEvent(this.id)
	Recount.db.char.RealtimeWindows[this.id]=nil
end
function me:CreateRealtimeWindow(who,tracking,ending)
	local theFrame=Recount:CreateFrame(nil,"",232,200,nil, me.FreeWindow)

	theFrame:SetResizable(true)

	theFrame:SetMinResize(150,64)
	theFrame:SetMaxResize(400,432)	

	theFrame:SetScript("OnSizeChanged", function()
						if ( this.isResizing ) then
							me.ResizeRealtimeWindow(this)
							
						end
					end)

	if string.sub(who,1,1)~="!" then
		theFrame.TitleText=who..ending
	else
		theFrame.TitleText=ending
	end

	theFrame.Title:SetText(theFrame.TitleText.." - 0.0")

	theFrame.DragBottomRight = CreateFrame("Frame", nil, theFrame)
	theFrame.DragBottomRight:Show()
	theFrame.DragBottomRight:SetFrameLevel( theFrame:GetFrameLevel() + 10)
	theFrame.DragBottomRight:SetWidth(16)
	theFrame.DragBottomRight:SetHeight(16)
	theFrame.DragBottomRight:SetPoint("BOTTOMRIGHT", theFrame, "BOTTOMRIGHT", 0, 0)
	theFrame.DragBottomRight:EnableMouse(true)
	theFrame.DragBottomRight:SetScript("OnMouseDown", function() this:GetParent().isResizing = true; this:GetParent():StartSizing("BOTTOMRIGHT") end )
	theFrame.DragBottomRight:SetScript("OnMouseUp", function() this:GetParent():StopMovingOrSizing(); this:GetParent().isResizing = false;this:GetParent():SavePosition() end )


	theFrame.DragBottomLeft = CreateFrame("Frame", nil, theFrame)
	theFrame.DragBottomLeft:Show()
	theFrame.DragBottomLeft:SetFrameLevel( theFrame:GetFrameLevel() + 10)
	theFrame.DragBottomLeft:SetWidth(16)
	theFrame.DragBottomLeft:SetHeight(16)
	theFrame.DragBottomLeft:SetPoint("BOTTOMLEFT", theFrame, "BOTTOMLEFT", 0, 0)
	theFrame.DragBottomLeft:EnableMouse(true)
	theFrame.DragBottomLeft:SetScript("OnMouseDown", function() this:GetParent().isResizing = true; this:GetParent():StartSizing("BOTTOMLEFT") end )
	theFrame.DragBottomLeft:SetScript("OnMouseUp", function() this:GetParent():StopMovingOrSizing(); this:GetParent().isResizing = false;this:GetParent():SavePosition() end )

	local g=Graph:CreateGraphRealtime("Recount_Realtime_"..who.."_"..tracking,theFrame,"BOTTOM","BOTTOM",0,2,197,199)
	g:SetAutoScale(true)
	g:SetGridSpacing(1.0,100)
	g:SetYMax(120)
	g:SetXAxis(-10,-0)
	g:SetMode("EXPFAST")
	g:SetDecay(0.5)
	g:SetFilterRadius(2)
	g:SetMinMaxY(100)
	g:SetBarColors({0.2,0.0,0.0,0.4},{1.0,0.0,0.0,1.0})
	g:SetUpdateLimit(0.05)
	g:SetGridColorSecondary({0.5,0.5,0.5,0.25})
	g:SetYLabels(true,true)
	g:SetGridSecondaryMultiple(1,2)

	
	theFrame.DetermineGridSpacing=me.DetermineGridSpacing
	theFrame.Graph=g
	
	theFrame.WindowNum=WindowNum
	WindowNum=WindowNum+1
	theFrame.id="Realtime_"..who.."_"..tracking.."_"..theFrame.WindowNum
	theFrame.who=who
	theFrame.ending=ending
	theFrame.tracking=tracking
	theFrame.SavePosition=me.SavePosition
	theFrame.ResizeRealtimeWindow=me.ResizeRealtimeWindow
	theFrame.UpdateTitle=me.UpdateTitle

	Recount.db.char.RealtimeWindows[theFrame.id]={who,tracking,ending}
	theFrame:StartMoving()
	theFrame:StopMovingOrSizing()
	theFrame:UpdateTitle()
	theFrame:SavePosition()
	

	Recount:RegisterTracking(theFrame.id,who,tracking,g.AddTimeData,g)

	--Need to add it to our window ordering system
	Recount:AddWindow(theFrame)

	Recount:ScheduleRepeatingEvent(theFrame.id,me.UpdateTitle,0.1,theFrame)

	return theFrame
end

function Recount:CreateRealtimeWindow(who,tracking,ending)
	local Window=table.maxn(FreeWindows)
	if Window>0 then
		if string.sub(who,1,1)~="!" then
			FreeWindows[Window].TitleText=who..ending
		else
			FreeWindows[Window].TitleText=ending
		end
		FreeWindows[Window].Title:SetText(FreeWindows[Window].TitleText.." - 0.0")
		FreeWindows[Window].id="Realtime_"..who.."_"..tracking.."_"..FreeWindows[Window].WindowNum
		FreeWindows[Window].who=who
		FreeWindows[Window].tracking=tracking
		FreeWindows[Window]:SetWidth(200)
		FreeWindows[Window]:SetHeight(232)
		me.ResizeRealtimeWindow(FreeWindows[Window])
		FreeWindows[Window]:ClearAllPoints()
		FreeWindows[Window]:SetPoint("CENTER",UIParent)
		FreeWindows[Window]:UpdateTitle()
		Recount:RegisterTracking(FreeWindows[Window].id,who,tracking,FreeWindows[Window].Graph.AddTimeData,FreeWindows[Window].Graph)
		Recount:ScheduleRepeatingEvent(FreeWindows[Window].id,me.UpdateTitle,0.1,FreeWindows[Window])
		FreeWindows[Window]:Show()

		Recount.db.char.RealtimeWindows[FreeWindows[Window].id]={who,tracking,ending}

		table.remove(FreeWindows,Window)
		return
	end

	me:CreateRealtimeWindow(who,tracking,ending)

	
end

function Recount:CreateRealtimeWindowSized(who,tracking,ending, x, y, width, height)
	local f=me:CreateRealtimeWindow(who,tracking,ending)

	f:SetPoint("TOPLEFT",UIParent,"TOPLEFT",x,y)
	f:SetWidth(width)
	f:SetHeight(height)
	f:ResizeRealtimeWindow()
	f:SavePosition()
end